	static Function parseRc($filename, $varname='$w', $classname='WbAppWindow', $parent=false, $notify="WBC_GETFOCUS | WBC_DBLCLICK"){
		// Open external file
		$rc = file_get_contents($filename);
		
		// Remove comments and useless spaces
		$rc = preg_replace("/^\s*;.*$/m", "", $rc);
		$rc = preg_replace("/^\s*(.*)$/m", "\\1", $rc);
		
		// Maintain #defines and discard the rest
		$def = preg_replace("/(?!^\s*#define)(.*)$/m", "\\2", $rc);
		
		// Remove blank lines
		$def = preg_replace("/\n+/m", "\n", $def);
		
		// Change string C #defines to PHP format
		$def = preg_replace("/#define\s+(\w+)\s+\"(.*)\"/", "if(!defined(\"\\1\")) define(\"\\1\", \"\\2\");", $def);
		
		// Change character C #defines to PHP format
		$def = preg_replace("/#define\s+(\w+)\s+'(.+)'/", "if(!defined(\"\\1\")) define(\"\\1\", \"\\2\");", $def);
		
		// Change numeric C #defines to PHP format
		$def = preg_replace("/#define\s+(\w+)\s+(\S+)/", "if(!defined(\"\\1\")) define(\"\\1\", \\2);", $def);
		$def = "// Control identifiers\n\n" . preg_replace("/(\r\n|\r|\n)+/sm", "\n", $def);
		
		// Return to original string and eliminates the #defines
		$rc = preg_replace("/^\s*#define(.*)$/m", "", $rc);
		
		/**
			Build main window.
			Regex info:
				1 = Dialog's ID
				2 = Dialog's left position
				3 = Dialog's top position
				4 = Dialog's width
				5 = Dialog's height
				6 = Dialog's caption
				7 = Dialog's style
				8 = Dialog's EXstyle
		**/
		$window_or_tabname = $varname;
		
		$controls = "// Creating main window:\r\n";
		if($classname == "WbAppWindow" || $classname == "WbPopupWindow" || $classname == "WbResizableWindow"){
			preg_match("/"."(.+?) DIALOGEX (.+),(.+),([0-9]{1,4}),([0-9]{1,4})\r?\n"."CAPTION \"(.*?)\"\r?\n"."FONT .+?\r?\n"."STYLE (0x[0-9A-Fa-f]+?)\r?\n"."EXSTYLE (0x[0-9A-Fa-f]+?)\r?\n"."/", $rc, $output);
			$window_or_tabname .= "[$output[1]]";
			$controls .= $window_or_tabname." = new $classname(\"" . addcslashes($output[6], '"$') . "\", $output[2], $output[3], $output[4], $output[5], $output[7] " . ((($notify)&&!($output[7]&WBC_NOTIFY))?'| WBC_NOTIFY':'') . ", $notify);";
		}
		elseif($classname == "WbModalDialog" || $classname == "WbModelessDialog" || $classname == "WbToolDialog"){
			$window_or_tabname .= "[$output[1]]";
			preg_match("/"."(.+?) DIALOGEX (.+),(.+),([0-9]{1,4}),([0-9]{1,4})\r?\n"."CAPTION \"(.*?)\"\r?\n"."FONT .+?\r?\n"."STYLE (0x[0-9A-Fa-f]+?)\r?\n"."EXSTYLE (0x[0-9A-Fa-f]+?)\r?\n"."/", $rc, $output);
			$controls .= $window_or_tabname." = new $classname($parent, \"" . addcslashes($output[6], '"$') . "\", $output[2], $output[3], $output[4], $output[5], $output[7] " . ((($notify)&&!($output[7]&WBC_NOTIFY))?'| WBC_NOTIFY':'') . ", $notify);";
		}
		else{
			$window_or_tabname = $classname;
			$controls .= "// Not creating any dialog.. Using '$varname' instead.\r\n";
		}
		
		$controls .= "\r\n";
		$controls .= "\r\n";
		$controls .= "// Creating controls into $varname\r\n";
		
		/**
			Build controls
			Regex info:
				1 = Caption
				2 = ID
				3 = Class
				4 = Style
				5 = PosX
				6 = PosY
				7 = Width
				8 = Height
				9 = ExStyle
		**/
		
		preg_match_all("/\"(.*?)\",(.*?),\"(.*?)\",(0x.*?),(.*?),(.*?),(.*?),(.*?),(.*?)\r?\n/", $rc, $output);
		for($x = 0; $x < sizeof($output[0]); $x++){
			$winclass = $output[3][$x];
			$winstyle = hexdec($output[4][$x]);
			$winexstyle = hexdec($output[9][$x]);
			
			if(_bit_test($winstyle, WS_VISIBLE))
				$style = "WBC_VISIBLE";
			else
				$style = "WBC_INVISIBLE";
			
			if(_bit_test($winstyle, WS_DISABLED))
				$style .= " | WBC_DISABLED";
			else
				$style .= " | WBC_ENABLED";
			
			if(_bit_test($winexstyle, WS_EX_STATICEDGE))
				$style .= " | WBC_BORDER";
			
			// Set attributes according to control class
			switch(strtolower($winclass)) {
				
				case 'button':
					
					switch($winstyle & 0x0F) {
						case BS_AUTORADIOBUTTON:
						case BS_RADIOBUTTON:
							$class = "RadioButton";
							if(_bit_test($winstyle, WS_GROUP))
								$style .= " | WBC_GROUP";
							break;
						case BS_AUTOCHECKBOX:
						case BS_CHECKBOX:
							$class = "CheckBox";
							break;
						case BS_GROUPBOX:
							$class = "Frame";
							break;
						case BS_PUSHBUTTON:
						default:
							$class = "PushButton";
							break;
					}
					break;
				
				case 'static':
					
					switch($winstyle & 0x1F) {
						case SS_ICON:
						case SS_BITMAP:
							$style .= " | WBC_IMAGE | WBC_CENTER";
							$class = "Frame";
							break;
						case SS_ETCHEDHORZ:
						case SS_ETCHEDVERT:
						case SS_ETCHEDFRAME:
							$class = "Frame";
							break;
						case SS_CENTER:
							if(_bit_test($winstyle, SS_WORDELLIPSIS))
								$style .= " | WBC_ELLIPSIS";
							$style .= " | WBC_CENTER";
							$class = "Label";
							break;
						case SS_RIGHT:
							if(_bit_test($winstyle, SS_WORDELLIPSIS))
								$style .= " | WBC_ELLIPSIS";
							$style .= " | WBC_RIGHT";
							$class = "Label";
							break;
						case SS_LEFT:
						default:
							if(!_bit_test($winstyle, SS_LEFTNOWORDWRAP))
								$style .= " | WBC_LINES";
							if(_bit_test($winstyle, SS_WORDELLIPSIS))
								$style .= " | WBC_ELLIPSIS";
							$class = "Label";
							break;
					}
					break;
				
				case 'edit':
					$class = "EditBox";
					if(_bit_test($winstyle, ES_MULTILINE)) {
						$style .= " | WBC_LINES";
					} else {
						switch($winstyle & 0x03) {
							case ES_CENTER:
								$style .= " | WBC_CENTER";
								break;
							case ES_RIGHT:
								$style .= " | WBC_RIGHT";
								break;
							case ES_LEFT:
							default:
								break;
						}
					}
					if(_bit_test($winstyle, ES_READONLY))
						$style .= " | WBC_READONLY";
					if(_bit_test($winstyle, ES_PASSWORD))
						$style .= " | WBC_MASKED";
					if(_bit_test($winstyle, ES_NUMBER))
						$style .= " | WBC_NUMBER";
					break;
				
				case 'richedit20a':
					if(_bit_test($winstyle, ES_READONLY))
						$style .= " | WBC_READONLY";
					$class = "RTFEditBox";
					switch($winstyle & 0x03) {
						case ES_CENTER:
							$style .= " | WBC_CENTER";
							break;
						case ES_RIGHT:
							$style .= " | WBC_RIGHT";
							break;
						case ES_LEFT:
						default:
							break;
					}
					break;
				
				case 'combobox':
					$class = "ComboBox";
					if(_bit_test($winstyle, CBS_SORT))
						$style .= " | WBC_SORT";
					break;
				
				case 'listbox':
					$class = "ListBox";
					if(_bit_test($winstyle, LBS_SORT))
						$style .= " | WBC_SORT";
					break;
				
				case 'scrollbar':
					$class = "ScrollBar";
					break;
				
				case 'syslistview32':
					$class = "ListView";
					if(!_bit_test($winstyle, LVS_NOSORTHEADER))
						$style .= " | WBC_SORT";
					if(_bit_test($winstyle, LVS_GRIDLINES))
						$style .= " | WBC_LINES";
					if(_bit_test($winstyle, LVS_CHECKBOXES))
						$style .= " | WBC_CHECKBOXES";
					break;
				
				case 'systabcontrol32':
					$class = "TabControl";
					break;
				
				case 'systreeview32':
					$class = "TreeView";
					break;
				
				case 'toolbarwindow32':
					$class = "ToolBar";
					break;
				
				case 'msctls_progress32':
					$class = "Gauge";
					break;
				
				case 'msctls_statusbar32':
					$class = "StatusBar";
					break;
				
				case 'msctls_trackbar32':
					$class = "Slider";
					if(_bit_test($winstyle, TBS_AUTOTICKS))
						$style .= " | WBC_LINES";
					break;
				
				case 'msctls_updown32':
					$class = "Spinner";
					break;
			}
			/**
				So, finally... Let's create it!
			**/
			$controls .= $varname."[".$output[2][$x]."] = ";
			$controls .= $window_or_tabname."->addControl('Wb$class', ".$output[5][$x].", ".$output[6][$x].",".$output[7][$x].",".$output[8][$x];
			$controls .= ($output[1][$x]?", \"".addcslashes($output[1][$x], '"$')."\"":'');
			$controls .= ", " . $output[2][$x];
			$controls .= ");\r\n";
		}
		return $def."\r\n\r\n".$controls;
	}
