/*******************************************************************************

 WINBINDER - A native Windows binding for PHP

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 Library of ZEND-specific functions for the WinBinder extension

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "phpwb.h"
//#include <stdio.h>
//#include <stdarg.h>			// For vprintf() and va_* macros

//-------------------------------------------------------------------- CONSTANTS

#define CALLBACK_ARGS	4	// Number of arguments of the callback function

//----------------------------------------------------------- EXPORTED FUNCTIONS

/*
	TODO: optionally use a message box to display errors and/or log errors to a file
*/

BOOL wbError(int nType, LPCTSTR pszFmt, ...)
{
	char szString[MAX_ERR_MESSAGE];

	va_list args;
	va_start(args, pszFmt);

	vsprintf(szString, pszFmt, args);
	va_end(args);

	switch(nType) {
		case MB_OK:
		case MB_ICONINFORMATION:
			nType = E_NOTICE;
			break;

		case MB_ICONWARNING:
			nType = E_WARNING;
			break;

		case MB_ICONSTOP:
		case MB_ICONQUESTION:
		default:
			nType = E_ERROR;
			break;
	}
	zend_error(nType, szString);
	return FALSE;
}

/*
 Call a user function.
 Reference: http://www.phpfreaks.com/forums/index.php?showtopic=10272
*/

BOOL wbCallUserFunction(LPCTSTR pszFunctionName, PWBOBJ pwboParent, PWBOBJ pctrl, UINT id, LPARAM lParam)
{
	zval *fname;
	zval *return_value = NULL;
	zval **parms[CALLBACK_ARGS];
	zval *z0, *z1, *z2, *z3;
	BOOL bRet;
	char *pszFName;
	TSRMLS_FETCH();

	pszFName = (char *)pszFunctionName;

	if(!pszFName || !*pszFName) {
		char szTitle[256];

		if(GetWindowText(pwboParent->handle, szTitle, 256))
			zend_error(E_WARNING, "No callback function assigned to window '%s'", szTitle);
		else
			zend_error(E_WARNING, "No callback function assigned to window #%ld", (LONG)pwboParent);
		return FALSE;
	}

	MAKE_STD_ZVAL(fname);
	ZVAL_STRING(fname, pszFName, 1);

	if(!zend_is_callable(fname, 0, &pszFName)) {
		zend_error(E_WARNING, "%s: '%s' is not a function or cannot be called",
		  __func__, pszFName);
		return FALSE;
	}

	ALLOC_INIT_ZVAL(z0);
	ZVAL_LONG(z0, (LONG)pwboParent);				// PWBOBJ pointer
	parms[0] = &z0;

	ALLOC_INIT_ZVAL(z1);							// id
	ZVAL_LONG(z1, (LONG)(id ? id : (pctrl->id ? pctrl->id : pwboParent->id)));
	parms[1] = &z1;

	ALLOC_INIT_ZVAL(z2);							// control handle
	ZVAL_LONG(z2, (LONG)pctrl);
	parms[2] = &z2;

	ALLOC_INIT_ZVAL(z3);							// lparam
	ZVAL_LONG(z3, (LONG)lParam);
	parms[3] = &z3;

	// Call the user function

//	printf("%s\n", fname->value.str.val);

	bRet = call_user_function_ex(
		CG(function_table),			// Hash value for the function table
		NULL,						// Pointer to an object (may be NULL)
		fname,						// Function name
		&return_value,				// Return value
		CALLBACK_ARGS,				// Parameter count
		parms,						// Parameter array
		0,							// No separation flag (always 0)
		NULL TSRMLS_CC
		);

	if(bRet != SUCCESS) {
    	zend_error(E_WARNING, "Function call failed");
	}
	zval_ptr_dtor(&return_value);
	return TRUE;
}

// Memory-allocation functions

void *wbAlloc(size_t size)
{
	return emalloc(size);
}

void *wbCalloc(size_t nmemb, size_t size)
{
	return ecalloc(nmemb, size);
}

void wbFree(void *ptr)
{
	if(ptr)
		efree(ptr);
}

void *wbRealloc(void *ptr, size_t size)
{
	return erealloc(ptr, size);
}

char *wbStrDup(const char *string)
{
	return estrdup(string);
}

char *wbStrnDup(const char *string, size_t size)
{
	return estrndup(string, size);
}

//------------------------------------------------------------------ END OF FILE
