/*******************************************************************************

 WINBINDER - A native Windows binding for PHP

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 ZEND wrapper for window creation and manipulation functions

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "phpwb.h"

//----------------------------------------------------------- EXPORTED FUNCTIONS

ZEND_FUNCTION(wb_create_window)
{
    long wbclass, x, y, w, h, style, lparam;
	char *caption = "";
	int caption_len;
	long pwboparent, pwbo;

	// Default parameter values

	x = WBC_CENTER;
	y = WBC_CENTER;
	w = CW_USEDEFAULT;
	h = CW_USEDEFAULT;
	style = 0;
	lparam = 0;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l!l|sllllll", &pwboparent, &wbclass, &caption, &caption_len, &x, &y, &w, &h, &style, &lparam) == FAILURE)
		return;

	if(pwboparent && !wbIsWBObj((void *)pwboparent))
		RETURN_NULL()

	if(ZEND_NUM_ARGS() == 5) {
		w = x;
		h = y;
		x = WBC_CENTER;
		y = WBC_CENTER;
	}

	pwbo = (long)wbCreateWindow((PWBOBJ)pwboparent, wbclass, caption, x, y, w, h, style, lparam);
	RETURN_LONG(pwbo);
}


ZEND_FUNCTION(wb_destroy_window)
{
	long pwbo;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()

	RETURN_BOOL(wbDestroyWindow((PWBOBJ)pwbo))
}

/* Returns an array with the width and height */

ZEND_FUNCTION(wb_get_size)
{
	long pwbo;
	DWORD size;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()

	// Build the array

	size = wbGetWindowSize((PWBOBJ)pwbo);
	array_init(return_value);
	add_next_index_long(return_value, LOWORD(size));
	add_next_index_long(return_value, HIWORD(size));
}

ZEND_FUNCTION(wb_set_size)
{
	long pwbo;
    long w, h;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "lll", &pwbo, &w, &h) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()

	RETURN_BOOL(wbSetWindowSize((PWBOBJ)pwbo, w, h));
}

/* Returns an array with the x and y positions */

ZEND_FUNCTION(wb_get_position)
{
	long pwbo;
	DWORD pos;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "l", &pwbo) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()

	// Build the array

	pos = wbGetWindowPosition((PWBOBJ)pwbo, NULL);
	array_init(return_value);
	add_next_index_long(return_value, LOWORD(pos));
	add_next_index_long(return_value, HIWORD(pos));
}

ZEND_FUNCTION(wb_set_position)
{
	long pwbo;
    long x, y;

	// Default parameter values
	x = WBC_CENTER;
	y = WBC_CENTER;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "l|ll", &pwbo, &x, &y) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL()

	RETURN_BOOL(wbSetWindowPosition((PWBOBJ)pwbo, x, y, NULL));
}

ZEND_FUNCTION(wb_set_handler)
{
	long pwbo;
	char *handler;
	int s_len;
	zval *name;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	 "ls", &pwbo, &handler, &s_len) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwbo))
		RETURN_NULL();

	MAKE_STD_ZVAL(name);
	ZVAL_STRING(name, handler, 1);

	if(!zend_is_callable(name, 0, &handler)) {
		zend_error(E_WARNING, "%s: '%s' is not a function or cannot be called",
		  __func__, handler);
		RETURN_NULL();
	} else {
		RETURN_BOOL(wbSetWindowHandler((PWBOBJ)pwbo, handler));
	}
}

ZEND_FUNCTION(wb_get_item_list)
{
	long pwboparent;
	PWBOBJ *plist;
	int nctrls, i;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
	  "l", &pwboparent) == FAILURE)
		return;

	if(!wbIsWBObj((void *)pwboparent))
		RETURN_NULL()

	// Build the array

	array_init(return_value);

	nctrls = wbGetControlList((PWBOBJ)pwboparent, NULL, 32767);
	if(nctrls) {
		plist = emalloc(nctrls * sizeof(PWBOBJ));
		wbGetControlList((PWBOBJ)pwboparent, plist, nctrls);
		for(i = 0; i < nctrls; i++)
			add_next_index_long(return_value, (LONG)plist[i]);
		efree(plist);
	}
}

//------------------------------------------------------------------ END OF FILE
