/*******************************************************************************

 WINBINDER - A native Windows binding

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 ListView control

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "wb.h"

//----------------------------------------------------------- EXPORTED FUNCTIONS

int wbCreateListViewItem(PWBOBJ pwbo, int nItem, int nImage, LPCTSTR pszText)
{
	int count;
	LV_ITEM lvi;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	count = (int)SendMessage(pwbo->handle, LVM_GETITEMCOUNT, 0, 0);
	if(nItem < 0)
		nItem = count;

	lvi.mask = (nImage >= 0 ? (LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM) : (LVIF_TEXT | LVIF_PARAM));
//	lvi.mask = (nImage >= 0 ? (LVIF_TEXT | LVIF_IMAGE) : LVIF_TEXT);
	lvi.iItem = nItem;
	lvi.iSubItem = 0;
	lvi.state = 0;
	lvi.stateMask = 0;
	lvi.pszText = (LPTSTR)((pszText && *pszText) ? pszText : NULL);
	lvi.cchTextMax = 0;
	lvi.iImage = ((nImage >= 0) ? nImage : 0);
	lvi.lParam = nItem;			// For LVM_SORTITEMS / CompareLVItems()

	return ListView_InsertItem(pwbo->handle, &lvi);
}

BOOL wbDeleteListViewItem(PWBOBJ pwbo, int nItem)
{
	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	return ListView_DeleteItem(pwbo->handle, nItem);
}

/* Create or destroy an ImageList for the ListView */

BOOL wbCreateListViewImages(PWBOBJ pwbo, HBITMAP hbmImage, int nImages, COLORREF clTransparent)
{
	HBITMAP hbmMask;
	static HIMAGELIST hi;
	BITMAP bm;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	if(hbmImage && nImages) {
		GetObject(hbmImage, sizeof(BITMAP), (LPSTR) &bm);
		if((bm.bmWidth == 0) | (bm.bmHeight == 0))
			return FALSE;
		if(clTransparent != CLR_INVALID) {
			hbmMask = wbCreateMask(hbmImage, clTransparent);
			hi = ImageList_Create(bm.bmWidth / nImages, bm.bmHeight, ILC_COLORDDB | ILC_MASK, nImages, 0);
			ImageList_Add(hi, hbmImage, hbmMask);
			ListView_SetImageList(pwbo->handle, hi, LVSIL_SMALL);
			DeleteObject(hbmMask);
		} else {
			hi = ImageList_Create(bm.bmWidth / nImages, bm.bmHeight, ILC_COLORDDB, nImages, 0);
			ImageList_Add(hi, hbmImage, NULL);
			ListView_SetImageList(pwbo->handle, hi, LVSIL_SMALL);
		}
	} else {
		ListView_SetImageList(pwbo->handle, NULL, LVSIL_SMALL);
		ImageList_Destroy(hi);
	}
	return TRUE;
}

BOOL wbGetListViewItemChecked(PWBOBJ pwbo, int nItem)
{
	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	return (ListView_GetItemState(pwbo->handle, nItem, LVIS_STATEIMAGEMASK) & LVIS_CHECKED);
}

BOOL wbSetListViewItemChecked(PWBOBJ pwbo, int nItem, BOOL bChecked)
{
	LV_ITEM lvi;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	lvi.stateMask = LVIS_STATEIMAGEMASK;
	lvi.state = bChecked ? LVIS_CHECKED : 0;

	return SendMessage(pwbo->handle, LVM_SETITEMSTATE, (WPARAM)nItem, (LPARAM)&lvi);
}

// Returns the number of checked items. Optionally fills the array pbItems
// with the checked items indices.

int wbGetListViewCheckedItems(PWBOBJ pwbo, int *pbItems)
{
	int i, nCount, nChecked;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	nCount = SendMessage(pwbo->handle, LVM_GETITEMCOUNT, 0, 0);
	if(!nCount)
		return 0;

	if(pbItems) {
		for(i = 0, nChecked = 0; i < nCount; i++) {
			if(ListView_GetItemState(pwbo->handle, i, LVIS_CHECKED))
				pbItems[nChecked++] = i;
		}
	} else {
		for(i = 0, nChecked = 0; i < nCount; i++) {
			if(ListView_GetItemState(pwbo->handle, i, LVIS_CHECKED))
				nChecked++;
		}
	}
	return nChecked;
}

// Returns the number of selected items. Optionally fills the array pbItems
// with the selected item indices.

int wbGetListViewSelectedItems(PWBOBJ pwbo, int *pbItems)
{
	int i, nCount, nSel;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	nCount = SendMessage(pwbo->handle, LVM_GETITEMCOUNT, 0, 0);
	if(!nCount)
		return 0;

//	nSel = SendMessage(pwbo->handle, LVM_GETSELECTEDCOUNT, 0, 0);
//	if(!nSel)
//		return 0;

	if(pbItems) {
		for(i = 0, nSel = 0; i < nCount; i++) {
			if(ListView_GetItemState(pwbo->handle, i, LVIS_SELECTED))
				pbItems[nSel++] = i;
		}
	} else {
		nSel = SendMessage(pwbo->handle, LVM_GETSELECTEDCOUNT, 0, 0);
	}
	return nSel;
}

/* Determine the number of columns in the pwbo control, optionally filling up an
  array of integers with column widths */

int wbGetListViewColumns(PWBOBJ pwbo, int *pwidths)
{
 	LV_COLUMN lvct;
	int i;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	for(i = 0; i < MAX_LISTVIEWCOLS; i++) {
		lvct.iSubItem = i - 0;
		lvct.mask = LVCF_SUBITEM;
		if(pwidths) {
			*(pwidths + i) = ListView_GetColumnWidth(pwbo->handle, i);
		}
		if(!ListView_GetColumn(pwbo->handle, i, &lvct))
			break;
	}
	return i;
}

BOOL wbSetListViewColumnWidths(PWBOBJ pwbo, int *pwidths)
{
 	LV_COLUMN lvct;
	int i;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	if(!pwidths)
		return FALSE;

	for(i = 0; i < MAX_LISTVIEWCOLS; i++) {
		lvct.iSubItem = i - 0;
		lvct.mask = LVCF_SUBITEM;
		if(pwidths) {
			;//*(pwidths + i) = 0;
		}
		if(!ListView_GetColumn(pwbo->handle, i, &lvct))
			break;
		if(!wbSetListViewColumnWidth(pwbo, i, pwidths[i]))
			return FALSE;
	}
	return TRUE;
}

/* If nWidth is negative, calculate width automatically */

BOOL wbSetListViewColumnWidth(PWBOBJ pwbo, int nCol, int nWidth)
{

//	LV_COLUMN lvc;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	if(nWidth < 0) {
//		SIZE siz;

//		wbGetTextSize(&siz, lpszItem);
//		nWidth = siz.cx + 14;	//****** 14 is empirical and arbitrary
		nWidth = LVSCW_AUTOSIZE;
	}

//	lvc.mask = LVCF_FMT | LVCF_TEXT | LVCF_SUBITEM | LVCF_WIDTH;
//	lvc.fmt = LVCFMT_LEFT;
//	lvc.cx = nWidth;

//	nCol = wbGetListViewColumns(pwbo);
//	lvc.iSubItem = nCol;
//	lvc.pszText  = lpszItem;
	ListView_SetColumnWidth(pwbo->handle, nCol, nWidth);

	return TRUE;
}

int wbGetListViewItemCount(PWBOBJ pwbo)
{
	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	return SendMessage(pwbo->handle, LVM_GETITEMCOUNT, 0, 0);
}

BOOL wbSetListViewItemImage(PWBOBJ pwbo, int nItem, int nCol, int nImage)
{
	LV_ITEM lvi;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	lvi.mask = LVIF_IMAGE | LVIF_STATE | LVIF_TEXT;
	lvi.iItem = nItem;
	lvi.iSubItem = nCol;
	lvi.state = 0;
	lvi.stateMask = 0;
	lvi.pszText = NULL;
	lvi.cchTextMax = 0;
	lvi.iImage = nImage;
	lvi.lParam = 0;

	return ListView_SetItem((HWND)pwbo->handle, &lvi);
}

BOOL wbClearListViewColumns(PWBOBJ pwbo)
{
	int i, nCols;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	nCols = wbGetListViewColumns(pwbo, NULL) - 1;

	// Must delete in reverse, otherwise it won't work

	for(i = nCols; i >= 0; i--)
		ListView_DeleteColumn(pwbo->handle, i);
	return TRUE;
}

/* If nWidth is negative, calculate width automatically */

BOOL wbCreateListViewColumn(PWBOBJ pwbo, int nCol, LPTSTR lpszItem, int nWidth)
{
	// Adapted from Dave's Book of Top Ten Lists for Great Windows Programming,
	// By Dave Edson, p. 521

	LV_COLUMN lvc;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	if(nWidth < 0) {
		SIZE siz;

		wbGetTextSize(&siz, lpszItem);
		nWidth = siz.cx + 14;	//****** 14 is empirical and arbitrary
	}

	lvc.mask = LVCF_FMT | LVCF_TEXT | LVCF_SUBITEM | LVCF_WIDTH;
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = nWidth;

	nCol = wbGetListViewColumns(pwbo, NULL);
	lvc.iSubItem = nCol;
	lvc.pszText  = lpszItem;
	return (ListView_InsertColumn(pwbo->handle, nCol, &lvc) != -1);


//	bRet = ListView_InsertColumn(pwbo->handle, nCol, &lvc);
//	ListView_SetColumnWidth(pwbo->handle, nCol, LVSCW_AUTOSIZE_USEHEADER); // Doesn't work
//	return (bRet != -1);
}

BOOL wbSelectListViewItem(PWBOBJ pwbo, int nItem, BOOL bState)
{
	LV_ITEM lvi;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	lvi.mask = LVIF_STATE;
	lvi.iItem = nItem;
	lvi.iSubItem = 0;
	lvi.state = bState ? LVIS_SELECTED : 0;
	lvi.stateMask = LVIS_SELECTED;
	lvi.pszText = NULL;
	lvi.cchTextMax = 0;
	lvi.iImage = 0;
	lvi.lParam = 0;

	return SendMessage(pwbo->handle, LVM_SETITEMSTATE, nItem, (LPARAM)&lvi);
}

/* Selects / deselects all items */

BOOL wbSelectAllListViewItems(PWBOBJ pwbo, BOOL bState)
{
	int i, count;
	LV_ITEM lvi;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	lvi.mask = LVIF_STATE;
	lvi.iSubItem = 0;
	lvi.state = bState ? LVIS_SELECTED : 0;
	lvi.stateMask = LVIS_SELECTED;
	lvi.pszText = NULL;
	lvi.cchTextMax = 0;
	lvi.iImage = 0;
	lvi.lParam = 0;

	count = ListView_GetItemCount(pwbo->handle);

	for(i = 0; i < count; i++) {
		lvi.iItem = i;
		if(!SendMessage(pwbo->handle, LVM_SETITEMSTATE, i, (LPARAM)&lvi))
			return FALSE;
	}
	return TRUE;
}

BOOL wbSetListViewItemText(PWBOBJ pwbo, int nItem, int nCol, LPCTSTR pszText)
{
	LV_ITEM lvi;

	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	lvi.mask = (LVIF_TEXT | LVIF_PARAM);
	lvi.iSubItem = nCol;
	lvi.pszText = (LPTSTR)pszText;
	lvi.lParam = (LPARAM)pszText;

	return SendMessage(pwbo->handle, LVM_SETITEMTEXT, nItem, (LPARAM)&lvi);
}

/* Return the text of a single cell */

BOOL wbGetListViewItemText(PWBOBJ pwbo, int nItem, int nCol, LPTSTR pszText, int nChar)
{
	if(!pwbo || !pwbo->handle || !IsWindow(pwbo->handle))
		return FALSE;

	if(!pszText || !nChar)
		return FALSE;

	if(nCol + 1 > wbGetListViewColumns(pwbo, NULL))		// Does the column exist?
		return FALSE;

	ListView_GetItemText(pwbo->handle, nItem, nCol, pszText, nChar);

	return (*pszText != '\0');
}

//------------------------------------------------------------------ END OF FILE
